@@ -195,7 +195,7 @@ floating characters to be added.
 
 =back
 
-3)  @^date/time-picture-string^ (Date / Time Conversions):
+3)  @^date/time-picture-string[^data-picture-string]^ (Date / Time Conversions):
 
 =over 4
 
@@ -222,11 +222,12 @@ A valid date string will be formatted / unformatted based on the I<format-string
 B<Date::Fmtstr2time> and B<Date::Time2fmtstr> are installed, the "valid date string" being 
 processed by B<fmt>() can be, and the output produced by B<unfmt>() will be a Perl/Unix time 
 integer.  Otherwise, the other valid data strings processed by B<fmt>() are 
-"yyyymmdd [hhmmss]", "mm-dd-yyyy [hh:mm:ss]", etc.  B<unfmt>() will return 
-"yyyymmdd [hhmm[ss]" unless B<Date::Time2fmtstr> is installed, in which case, it returns 
-a Perl/Unix time integer.  This can be changed using B<-outfmt>.  NOTE:  It is highly 
-recommended that both of these modules be installed if formatting or unformatting 
-date / time values, as the manual workarounds used do not always produce desired results.
+"yyyymmdd[ hhmmss]", "mm-dd-yyyy [hh:mm:ss]", etc.  B<unfmt>() will return 
+"yyyymmdd[ hhmm[ss]" unless B<Date::Time2fmtstr> is installed, in which case, it returns 
+a Perl/Unix time integer.  This can be changed specifying either B<-outfmt> or a 
+I<data-picture-string>.  NOTE:  It is highly recommended that both of these modules be 
+installed if formatting or unformatting date / time values, as the manual workarounds used 
+do not always produce desired results.
 
 Examples:
 
@@ -244,6 +245,13 @@ installed, in which case it will return 1420696800 (equivalent to "2015/01/08 00
 unfmt('@^mm-dd-yy^, '01-08-15', {-outfmt => 'yyyymmdd'}) will always return "20150108", 
 if B<Date::Time2fmtstr> is also installed.
 
+unfmt('@^mm-dd-yy^yyyymmdd^, '01-08-15') works the same way, always returning "20150108", 
+if B<Date::Time2fmtstr> is also installed.
+
+NOTE:  If using B<unfmt>() with either a I<data-picture-string> or I<-outfmt> is specified, 
+and B<Date::Time2fmtstr> is not installed, then I<data-picture-string> or I<-outfmt> must be 
+set to "yyyymmdd[hhmm[ss]]" or it will fail.
+
 =back
 
 4)  Regex substitution:
@@ -404,10 +412,31 @@ B<-bad> => '<char>' (default '*') - The character to fill the output string if t
 output string exceeds the specified maximum length and <-truncate> => 'error' is 
 specified.
 
+B<-infmt> => I<format-string> (default '') - Alternate format to expect the incoming 
+data to be in.  If a I<data-picture-string>, it overrides this option.  If specified, 
+in a B<fmt>() call, it causes input data to be read in in this format layout (before 
+being formatted by the I<format-string>) and returned.  Otherwise (if neither this 
+option nor a I<data-picture-string> is specified), the data can be in a variety of 
+layouts that B<fmt>() can recognize.  This option is not particularly useful 
+except for some additional error-checking, and generally need not be used.
+
+NOTE:  If this option is specified, and B<Date::Fmtstr2time> is not installed, then 
+it must be set to "yyyymmdd[hhmm[ss]]" or the format will fail.
+
 B<-nonnumeric> => true | false (default false or 0) - whether or not to ignore 
 "numeric"-specific formatting, ie. adding commas, sign indicators, decimal places, 
 etc. even if the data is "numeric".
 
+B<-outfmt> => I<format-string> (default '') - Alternate format to return the 
+"unformatted" result in.  If a I<data-picture-string>, it overrides this option.  
+If specified in a B<unfmt>() call, it causes the result to be formatted according to 
+this format (after being unformatted by the I<format-string>) and returned.  
+Otherwise (if not specified), the result is returned as a Perl / Unix Time integer 
+(if B<Date::Fmtstr2time> is installed) or in "yyyymmdd[hhmm[ss]]" format if not.
+
+NOTE:  If this option is specified, and B<Date::Time2fmtstr> is not installed, then 
+it must be set to "yyyymmdd[hhmm[ss]]" or the unformat will fail.
+
 B<-sizefixed> => true | false (default false or 0) - If true, prevents expansion of 
 certain numeric formats when the number is positive or more than one comma is added.  
 What it actually does is set the format size to be fixed to the value returned by 
@@ -481,7 +510,7 @@ package String::PictureFormat;
 use strict;
 #use warnings;
 use vars qw(@ISA @EXPORT $VERSION);
-$VERSION = '1.01';
+$VERSION = '1.1';
 
 use Time::Local;
 
@@ -565,9 +594,26 @@ sub fmt {
 			return wantarray ? ($v, $leni, $justify) : $v;
 		} elsif ($pic =~ s#^\^##o) {  #DATE-CONVERSION
 			eval 'use Date::Time2fmtstr; $haveTime2fmtstr = 1; 1'  unless ($haveTime2fmtstr);
-			$suffix = ($pic =~ s#\^(.*)$##) ? $1 : '';
+			$pic =~ s/\\\^/\x04/go;   #PROTECT ESCAPED "^" IN FORMAT STRING!
+			$suffix = ($pic =~ s#\^([^\^]*)$##) ? $1 : '';
+			$suffix =~ s/\x04/\^/go;  #UNPROTECT ESCAPED "^" IN FORMAT STRING!
+			my $inpic = '';
+			($pic, $inpic) = split(/\^/, $pic)  if ($pic =~ /\^/);
+			$pic =~ s/\x04/\^/go;     #UNPROTECT ESCAPED "^" IN FORMAT STRING!
+			$inpic ||= $ops->{'-infmt'}  if ($ops->{'-infmt'});
+			my $perltime = 0;
+			if ($inpic) {
+				$inpic =~ s/\x04/\^/go;  #UNPROTECT ESCAPED "^" IN FORMAT STRING!
+				eval 'use Date::Fmtstr2time; $haveFmtstr2time = 1; 1'  unless ($haveFmtstr2time);
+				$perltime = str2time($v, $inpic)  if ($haveFmtstr2time);
+				unless ($perltime || (length($v) == length($inpic) && $inpic =~ /^yyyymmdd(?:hhmm(?:ss)?)?$/i)) {
+					$leni ||= $fixedLeni || length($inpic);
+					$v = $errchar x $leni;
+					return wantarray ? ($v, $leni, $justify) : $v;
+				}
+			}
 			my $t;
-			my $perltime = ($v =~ /^\d{9,11}$/o) ? $v : 0;
+			$perltime ||= ($v =~ /^\d{9,11}$/o) ? $v : 0;
 			unless ($perltime) {  #WE HAVE A DATE STRING, IE. yyyy-dd-mm, etc. THAT CHKDATE CAN HANDLE:
 				($t, $perltime) = _chkdate($v);
 				unless ($t || $perltime) {
@@ -578,11 +624,10 @@ sub fmt {
 				if ($haveTime2fmtstr) {
 					$v = $perltime || &timelocal(0,0,0,substr($t,6,2),
 							(substr($t,4,2)-1),substr($t,0,4),0,0,0);
-				} elsif ($perltime) {
-					$v = $perltime;
 				}
 			}
-			if ($v =~ /^\d{8}\d+$/o) {   #WE HAVE A PERL "TIME":
+			$v = $perltime  if ($perltime);
+			if ($perltime) {   #WE HAVE A PERL "TIME":
 				if ($haveTime2fmtstr) {       #WE ALSO HAVE Time2fmtstr!:
 					$t = time2str($v, $pic);
 					$leni ||= $fixedLeni || length($t);
@@ -599,7 +644,7 @@ sub fmt {
 					$t = sprintf('%4.4d',$tv[5]+1900) . sprintf('%2.2d',$tv[4]+1) . sprintf('%2.2d',$tv[3])
 							. ' ' . sprintf('%2.2d',$tv[2]) . sprintf('%2.2d',$tv[1]) . sprintf('%2.2d',$tv[0]);
 				}
-			} 
+			}
 			if ($t =~ /^\d{8}(?: \d{4,6})?$/o) {   #WE HAVE A STRING WE CAN TRY TO CONVERT MANUALLY:
 				$pic =~ s/yyyy/substr($t,0,4)/ie;
 				$pic =~ s/yy/substr($t,2,4)/ie;
@@ -610,8 +655,8 @@ sub fmt {
 				$pic =~ s/ss/substr($t,13,2)/ie;
 				$v = $pic;
 				$leni ||= $fixedLeni || length($v);
-				if ($leni && length($t) > $leni && $ops->{'-truncate'} !~ /no/io) {
-					$v = ($ops->{'-truncate'} =~ /er/io) ? $errchar x $fixedLeni 
+				if ($leni && length($v) > $leni && $ops->{'-truncate'} !~ /no/io) {
+					$v = ($ops->{'-truncate'} =~ /er/io) ? $errchar x $leni 
 							: substr($t, 0, $fixedLeni);
 				}
 				$v .= $suffix  unless ($ops->{'-suffix'} =~ /no/io);
@@ -966,7 +1011,7 @@ sub unfmt {
 		$leni = $fixedLeni  if ($fixedLeni);
 		$leni = $1  if ($pic =~ s/^(\d+)\://o);
 		$leniSpecified = $leni;
-		if ($pic =~ s/^([\'\"\/\`])//o) {         #PICTURE LITERAL   (@'foo'
+		if ($pic =~ s/^([\'\"\/\`])//o) {         #PICTURE LITERAL (@'foo'
 			my $regexDelimiter = $1;         #REPLACE EACH DOT WITH NEXT CHAR. SKIP ONES CORRESPONDING WITH "^", ALL OTHER CHARS ARE LITERAL.
 			$v =~ s/$1$//  if ($pic =~ s#\Q$regexDelimiter\E(.*)$##);
 			my $r0 = $pic;
@@ -993,14 +1038,27 @@ sub unfmt {
 			return wantarray ? ($v, $leni, $justify) : $v;
 		} elsif ($pic =~ s#^\^##o) {  #DATE-CONVERSION
 			eval 'use Date::Fmtstr2time; $haveFmtstr2time = 1; 1'  unless ($haveFmtstr2time);
+			$pic =~ s/\\\^/\x04/go;   #PROTECT ESCAPED "^" IN FORMAT STRING!
+			$suffix = ($pic =~ s#\^([^\^]*)$##) ? $1 : '';
+			$suffix =~ s/\x04/\^/go;  #UNPROTECT ESCAPED "^" IN FORMAT STRING!
+			my $outpic = '';
+			($pic, $outpic) = split(/\^/o, $pic)  if ($pic =~ /\^/);
+			$pic =~ s/\x04/\^/go;     #UNPROTECT ESCAPED "^" IN FORMAT STRING!
+			$outpic ||= $ops->{'-outfmt'}  if ($ops->{'-outfmt'});
+			$v =~ s/\Q${suffix}\E$//  unless ($ops->{'-suffix'} =~ /no/io);
 			my $t = '';
 			if ($haveFmtstr2time) {   #CONVERT TO A PERL "TIME" USING Fmtstr2time IF IT'S AVAILABLE:
 				$t = str2time($v, $pic);
-				if ($t =~ /^\d+$/o && $ops->{'-outfmt'}) {  #WE WANT THE TIME FORMATTED TO A STRING:
+				if ($t && $outpic) {  #WE WANT THE TIME FORMATTED TO A STRING:
 					eval 'use Date::Time2fmtstr; $haveTime2fmtstr = 1; 1'  unless ($haveTime2fmtstr);
-					$t = time2str($t, $ops->{'-outfmt'})  if ($haveTime2fmtstr);
+					$t = ($haveTime2fmtstr) ? time2str($t, $outpic) : '';
+				}
+			} 
+			unless ($t) {      #ATTEMPT A MANUAL TRANSLATION TO AN INTEGER FORMATTED:  yyyymmdd[hhmm[ss]]
+				if ($outpic && $outpic !~ /^yyyymmdd(?:hhmm(?:ss)?)?$/i) {  #IF WE DON'T HAVE Fmtstr2time & user specified an output format other than "yyyymmdd..." then FAIL!
+					$t = $errchar x length($outpic);
+					return wantarray ? ($t, $leni, $justify) : $t;
 				}
-			} else {      #ATTEMPT A MANUAL TRANSLATION TO AN INTEGER FORMATTED:  yyyymmdd[hhmm[ss]]
 				foreach my $i (qw(yyyy mm dd)) {
 					$t .= substr($v,index($pic,$i),length($i)) || ' ' x length($i);
 				}
@@ -1241,9 +1299,8 @@ sub fmtsiz {
 			$pic =~ s/\\\^/\x04/go;
 			return length($pic);
 		} elsif ($pic =~ s#^\^##o) {  #DATE-CONVERSION
+			$pic =~ s/\\\^/\x04/go;   #PROTECT ESCAPED "^" IN FORMAT STRING!
 			$pic =~ s#\^.*$##;
-			$pic =~ s/\x03/\)/go;			
-			$pic =~ s/\x02/\(/go;			
 			(my $t = $v) =~ s/\D//go;
 			return length($pic);
 		} elsif ($pic =~ m#^(?:s|tr)\W#o) {         #REGEX SUBSTITUTION (@s/foo/bar/)
@@ -1389,7 +1446,9 @@ sub fmtsuffix {
 			$suffix = $1  if ($pic =~ s#\Q$regexDelimiter\E(.*)$##);
 			return $suffix;
 		} elsif ($pic =~ s#^\^##o) {         #DATE-CONVERSION
-			$suffix = $1  if ($pic =~ s#\^(.*)$##o);
+			$pic =~ s/\\\^/\x04/go;   #PROTECT ESCAPED "^" IN FORMAT STRING!
+			$suffix = ($pic =~ s#\^([^\^]*)$##) ? $1 : '';
+			$suffix =~ s/\x04/\^/go;  #UNPROTECT ESCAPED "^" IN FORMAT STRING!
 			return $suffix;
 		} elsif ($pic =~ m#^(?:s|tr)(\W)#) {          #REGEX SUBSTITUTION (@s/foo/bar/)
 			my $regexDelimiter = $1;
